# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MREDIT_METADATA = Metadata(
    id="678a737a8aca3b1d9299ee0e4e4b44fe21b937dd.boutiques",
    name="mredit",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MreditPlaneParamsDictNoTag = typing.TypedDict('_MreditPlaneParamsDictNoTag', {
    "axis": int,
    "coord": list[int],
    "value": float,
})
MreditPlaneParamsDictTagged = typing.TypedDict('MreditPlaneParamsDictTagged', {
    "@type": typing.Literal["plane"],
    "axis": int,
    "coord": list[int],
    "value": float,
})
MreditPlaneParamsDict = _MreditPlaneParamsDictNoTag | MreditPlaneParamsDictTagged


_MreditSphereParamsDictNoTag = typing.TypedDict('_MreditSphereParamsDictNoTag', {
    "position": list[float],
    "radius": float,
    "value": float,
})
MreditSphereParamsDictTagged = typing.TypedDict('MreditSphereParamsDictTagged', {
    "@type": typing.Literal["sphere"],
    "position": list[float],
    "radius": float,
    "value": float,
})
MreditSphereParamsDict = _MreditSphereParamsDictNoTag | MreditSphereParamsDictTagged


_MreditVoxelParamsDictNoTag = typing.TypedDict('_MreditVoxelParamsDictNoTag', {
    "position": list[float],
    "value": float,
})
MreditVoxelParamsDictTagged = typing.TypedDict('MreditVoxelParamsDictTagged', {
    "@type": typing.Literal["voxel"],
    "position": list[float],
    "value": float,
})
MreditVoxelParamsDict = _MreditVoxelParamsDictNoTag | MreditVoxelParamsDictTagged


_MreditConfigParamsDictNoTag = typing.TypedDict('_MreditConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MreditConfigParamsDictTagged = typing.TypedDict('MreditConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MreditConfigParamsDict = _MreditConfigParamsDictNoTag | MreditConfigParamsDictTagged


_MreditParamsDictNoTag = typing.TypedDict('_MreditParamsDictNoTag', {
    "plane": typing.NotRequired[list[MreditPlaneParamsDict] | None],
    "sphere": typing.NotRequired[list[MreditSphereParamsDict] | None],
    "voxel": typing.NotRequired[list[MreditVoxelParamsDict] | None],
    "scanner": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MreditConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": typing.NotRequired[str | None],
})
MreditParamsDictTagged = typing.TypedDict('MreditParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mredit"],
    "plane": typing.NotRequired[list[MreditPlaneParamsDict] | None],
    "sphere": typing.NotRequired[list[MreditSphereParamsDict] | None],
    "voxel": typing.NotRequired[list[MreditVoxelParamsDict] | None],
    "scanner": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MreditConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": typing.NotRequired[str | None],
})
MreditParamsDict = _MreditParamsDictNoTag | MreditParamsDictTagged


def mredit_plane(
    axis: int,
    coord: list[int],
    value: float,
) -> MreditPlaneParamsDictTagged:
    """
    Build parameters.
    
    Args:
        axis: fill one or more planes on a particular image axis.
        coord: fill one or more planes on a particular image axis.
        value: fill one or more planes on a particular image axis.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "plane",
        "axis": axis,
        "coord": coord,
        "value": value,
    }
    return params


def mredit_plane_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MreditPlaneParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("axis", None) is None:
        raise StyxValidationError("`axis` must not be None")
    if not isinstance(params["axis"], int):
        raise StyxValidationError(f'`axis` has the wrong type: Received `{type(params.get("axis", None))}` expected `int`')
    if params.get("coord", None) is None:
        raise StyxValidationError("`coord` must not be None")
    if not isinstance(params["coord"], list):
        raise StyxValidationError(f'`coord` has the wrong type: Received `{type(params.get("coord", None))}` expected `list[int]`')
    for e in params["coord"]:
        if not isinstance(e, int):
            raise StyxValidationError(f'`coord` has the wrong type: Received `{type(params.get("coord", None))}` expected `list[int]`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], (float, int)):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `float`')


def mredit_plane_cargs(
    params: MreditPlaneParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-plane")
    cargs.append(str(params.get("axis", None)))
    cargs.extend(map(str, params.get("coord", None)))
    cargs.append(str(params.get("value", None)))
    return cargs


def mredit_sphere(
    position: list[float],
    radius: float,
    value: float,
) -> MreditSphereParamsDictTagged:
    """
    Build parameters.
    
    Args:
        position: draw a sphere with radius in mm.
        radius: draw a sphere with radius in mm.
        value: draw a sphere with radius in mm.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "sphere",
        "position": position,
        "radius": radius,
        "value": value,
    }
    return params


def mredit_sphere_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MreditSphereParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("position", None) is None:
        raise StyxValidationError("`position` must not be None")
    if not isinstance(params["position"], list):
        raise StyxValidationError(f'`position` has the wrong type: Received `{type(params.get("position", None))}` expected `list[float]`')
    for e in params["position"]:
        if not isinstance(e, (float, int)):
            raise StyxValidationError(f'`position` has the wrong type: Received `{type(params.get("position", None))}` expected `list[float]`')
    if params.get("radius", None) is None:
        raise StyxValidationError("`radius` must not be None")
    if not isinstance(params["radius"], (float, int)):
        raise StyxValidationError(f'`radius` has the wrong type: Received `{type(params.get("radius", None))}` expected `float`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], (float, int)):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `float`')


def mredit_sphere_cargs(
    params: MreditSphereParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-sphere")
    cargs.extend(map(str, params.get("position", None)))
    cargs.append(str(params.get("radius", None)))
    cargs.append(str(params.get("value", None)))
    return cargs


def mredit_voxel(
    position: list[float],
    value: float,
) -> MreditVoxelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        position: change the image value within a single voxel.
        value: change the image value within a single voxel.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "voxel",
        "position": position,
        "value": value,
    }
    return params


def mredit_voxel_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MreditVoxelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("position", None) is None:
        raise StyxValidationError("`position` must not be None")
    if not isinstance(params["position"], list):
        raise StyxValidationError(f'`position` has the wrong type: Received `{type(params.get("position", None))}` expected `list[float]`')
    for e in params["position"]:
        if not isinstance(e, (float, int)):
            raise StyxValidationError(f'`position` has the wrong type: Received `{type(params.get("position", None))}` expected `list[float]`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], (float, int)):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `float`')


def mredit_voxel_cargs(
    params: MreditVoxelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-voxel")
    cargs.extend(map(str, params.get("position", None)))
    cargs.append(str(params.get("value", None)))
    return cargs


def mredit_config(
    key: str,
    value: str,
) -> MreditConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mredit_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MreditConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mredit_config_cargs(
    params: MreditConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MreditOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MreditParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType | None
    """the (optional) output image"""


def mredit_params(
    input_: InputPathType,
    plane: list[MreditPlaneParamsDict] | None = None,
    sphere: list[MreditSphereParamsDict] | None = None,
    voxel: list[MreditVoxelParamsDict] | None = None,
    scanner: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MreditConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    output: str | None = None,
) -> MreditParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image.
        plane: fill one or more planes on a particular image axis.
        sphere: draw a sphere with radius in mm.
        voxel: change the image value within a single voxel.
        scanner: indicate that coordinates are specified in scanner space,\
            rather than as voxel coordinates.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        output: the (optional) output image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mredit",
        "scanner": scanner,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
    }
    if plane is not None:
        params["plane"] = plane
    if sphere is not None:
        params["sphere"] = sphere
    if voxel is not None:
        params["voxel"] = voxel
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    if output is not None:
        params["output"] = output
    return params


def mredit_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MreditParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("plane", None) is not None:
        if not isinstance(params["plane"], list):
            raise StyxValidationError(f'`plane` has the wrong type: Received `{type(params.get("plane", None))}` expected `list[MreditPlaneParamsDict] | None`')
        for e in params["plane"]:
            mredit_plane_validate(e)
    if params.get("sphere", None) is not None:
        if not isinstance(params["sphere"], list):
            raise StyxValidationError(f'`sphere` has the wrong type: Received `{type(params.get("sphere", None))}` expected `list[MreditSphereParamsDict] | None`')
        for e in params["sphere"]:
            mredit_sphere_validate(e)
    if params.get("voxel", None) is not None:
        if not isinstance(params["voxel"], list):
            raise StyxValidationError(f'`voxel` has the wrong type: Received `{type(params.get("voxel", None))}` expected `list[MreditVoxelParamsDict] | None`')
        for e in params["voxel"]:
            mredit_voxel_validate(e)
    if params.get("scanner", False) is None:
        raise StyxValidationError("`scanner` must not be None")
    if not isinstance(params["scanner"], bool):
        raise StyxValidationError(f'`scanner` has the wrong type: Received `{type(params.get("scanner", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MreditConfigParamsDict] | None`')
        for e in params["config"]:
            mredit_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is not None:
        if not isinstance(params["output"], str):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str | None`')


def mredit_cargs(
    params: MreditParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mredit")
    if params.get("plane", None) is not None:
        cargs.extend([a for c in [mredit_plane_cargs(s, execution) for s in params.get("plane", None)] for a in c])
    if params.get("sphere", None) is not None:
        cargs.extend([a for c in [mredit_sphere_cargs(s, execution) for s in params.get("sphere", None)] for a in c])
    if params.get("voxel", None) is not None:
        cargs.extend([a for c in [mredit_voxel_cargs(s, execution) for s in params.get("voxel", None)] for a in c])
    if params.get("scanner", False):
        cargs.append("-scanner")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mredit_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    if params.get("output", None) is not None:
        cargs.append(params.get("output", None))
    return cargs


def mredit_outputs(
    params: MreditParamsDict,
    execution: Execution,
) -> MreditOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MreditOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)) if (params.get("output") is not None) else None,
    )
    return ret


def mredit_execute(
    params: MreditParamsDict,
    runner: Runner | None = None,
) -> MreditOutputs:
    """
    mredit
    
    Directly edit the intensities within an image from the command-line.
    
    A range of options are provided to enable direct editing of voxel
    intensities based on voxel / real-space coordinates. If only one image path
    is provided, the image will be edited in-place (use at own risk); if input
    and output image paths are provided, the output will contain the edited
    image, and the original image will not be modified in any way.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MreditOutputs`).
    """
    mredit_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MREDIT_METADATA)
    params = execution.params(params)
    cargs = mredit_cargs(params, execution)
    ret = mredit_outputs(params, execution)
    execution.run(cargs)
    return ret


def mredit(
    input_: InputPathType,
    plane: list[MreditPlaneParamsDict] | None = None,
    sphere: list[MreditSphereParamsDict] | None = None,
    voxel: list[MreditVoxelParamsDict] | None = None,
    scanner: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MreditConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    output: str | None = None,
    runner: Runner | None = None,
) -> MreditOutputs:
    """
    mredit
    
    Directly edit the intensities within an image from the command-line.
    
    A range of options are provided to enable direct editing of voxel
    intensities based on voxel / real-space coordinates. If only one image path
    is provided, the image will be edited in-place (use at own risk); if input
    and output image paths are provided, the output will contain the edited
    image, and the original image will not be modified in any way.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image.
        plane: fill one or more planes on a particular image axis.
        sphere: draw a sphere with radius in mm.
        voxel: change the image value within a single voxel.
        scanner: indicate that coordinates are specified in scanner space,\
            rather than as voxel coordinates.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        output: the (optional) output image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MreditOutputs`).
    """
    params = mredit_params(
        plane=plane,
        sphere=sphere,
        voxel=voxel,
        scanner=scanner,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return mredit_execute(params, runner)


__all__ = [
    "MREDIT_METADATA",
    "MreditConfigParamsDict",
    "MreditConfigParamsDictTagged",
    "MreditOutputs",
    "MreditParamsDict",
    "MreditParamsDictTagged",
    "MreditPlaneParamsDict",
    "MreditPlaneParamsDictTagged",
    "MreditSphereParamsDict",
    "MreditSphereParamsDictTagged",
    "MreditVoxelParamsDict",
    "MreditVoxelParamsDictTagged",
    "mredit",
    "mredit_config",
    "mredit_execute",
    "mredit_params",
    "mredit_plane",
    "mredit_sphere",
    "mredit_voxel",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TENSOR2METRIC_METADATA = Metadata(
    id="27f4e9c1379c830bcec96cfb3c8b321a7c99c245.boutiques",
    name="tensor2metric",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Tensor2metricConfigParamsDictNoTag = typing.TypedDict('_Tensor2metricConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Tensor2metricConfigParamsDictTagged = typing.TypedDict('Tensor2metricConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Tensor2metricConfigParamsDict = _Tensor2metricConfigParamsDictNoTag | Tensor2metricConfigParamsDictTagged


_Tensor2metricParamsDictNoTag = typing.TypedDict('_Tensor2metricParamsDictNoTag', {
    "adc": typing.NotRequired[str | None],
    "fa": typing.NotRequired[str | None],
    "ad": typing.NotRequired[str | None],
    "rd": typing.NotRequired[str | None],
    "cl": typing.NotRequired[str | None],
    "cp": typing.NotRequired[str | None],
    "cs": typing.NotRequired[str | None],
    "value": typing.NotRequired[str | None],
    "vector": typing.NotRequired[str | None],
    "num": typing.NotRequired[list[int] | None],
    "modulate": typing.NotRequired[str | None],
    "mask": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Tensor2metricConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tensor": InputPathType,
})
Tensor2metricParamsDictTagged = typing.TypedDict('Tensor2metricParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tensor2metric"],
    "adc": typing.NotRequired[str | None],
    "fa": typing.NotRequired[str | None],
    "ad": typing.NotRequired[str | None],
    "rd": typing.NotRequired[str | None],
    "cl": typing.NotRequired[str | None],
    "cp": typing.NotRequired[str | None],
    "cs": typing.NotRequired[str | None],
    "value": typing.NotRequired[str | None],
    "vector": typing.NotRequired[str | None],
    "num": typing.NotRequired[list[int] | None],
    "modulate": typing.NotRequired[str | None],
    "mask": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Tensor2metricConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tensor": InputPathType,
})
Tensor2metricParamsDict = _Tensor2metricParamsDictNoTag | Tensor2metricParamsDictTagged


def tensor2metric_config(
    key: str,
    value: str,
) -> Tensor2metricConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tensor2metric_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Tensor2metricConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tensor2metric_config_cargs(
    params: Tensor2metricConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Tensor2metricOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Tensor2metricParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    adc: OutputPathType | None
    """compute the mean apparent diffusion coefficient (ADC) of the diffusion
    tensor. (sometimes also referred to as the mean diffusivity (MD)) """
    fa: OutputPathType | None
    """compute the fractional anisotropy (FA) of the diffusion tensor. """
    ad: OutputPathType | None
    """compute the axial diffusivity (AD) of the diffusion tensor. (equivalent
    to the principal eigenvalue) """
    rd: OutputPathType | None
    """compute the radial diffusivity (RD) of the diffusion tensor. (equivalent
    to the mean of the two non-principal eigenvalues) """
    cl: OutputPathType | None
    """compute the linearity metric of the diffusion tensor. (one of the three
    Westin shape metrics) """
    cp: OutputPathType | None
    """compute the planarity metric of the diffusion tensor. (one of the three
    Westin shape metrics) """
    cs: OutputPathType | None
    """compute the sphericity metric of the diffusion tensor. (one of the three
    Westin shape metrics) """
    value: OutputPathType | None
    """compute the selected eigenvalue(s) of the diffusion tensor. """
    vector: OutputPathType | None
    """compute the selected eigenvector(s) of the diffusion tensor. """


def tensor2metric_params(
    tensor: InputPathType,
    adc: str | None = None,
    fa: str | None = None,
    ad: str | None = None,
    rd: str | None = None,
    cl: str | None = None,
    cp: str | None = None,
    cs: str | None = None,
    value: str | None = None,
    vector: str | None = None,
    num: list[int] | None = None,
    modulate: str | None = None,
    mask: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Tensor2metricConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Tensor2metricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tensor: the input tensor image.
        adc: compute the mean apparent diffusion coefficient (ADC) of the\
            diffusion tensor. (sometimes also referred to as the mean diffusivity\
            (MD)).
        fa: compute the fractional anisotropy (FA) of the diffusion tensor.
        ad: compute the axial diffusivity (AD) of the diffusion tensor.\
            (equivalent to the principal eigenvalue).
        rd: compute the radial diffusivity (RD) of the diffusion tensor.\
            (equivalent to the mean of the two non-principal eigenvalues).
        cl: compute the linearity metric of the diffusion tensor. (one of the\
            three Westin shape metrics).
        cp: compute the planarity metric of the diffusion tensor. (one of the\
            three Westin shape metrics).
        cs: compute the sphericity metric of the diffusion tensor. (one of the\
            three Westin shape metrics).
        value: compute the selected eigenvalue(s) of the diffusion tensor.
        vector: compute the selected eigenvector(s) of the diffusion tensor.
        num: specify the desired eigenvalue/eigenvector(s). Note that several\
            eigenvalues can be specified as a number sequence. For example, '1,3'\
            specifies the principal (1) and minor (3) eigenvalues/eigenvectors\
            (default = 1).
        modulate: specify how to modulate the magnitude of the eigenvectors.\
            Valid choices are: none, FA, eigval (default = FA).
        mask: only perform computation within the specified binary brain mask\
            image.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tensor2metric",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tensor": tensor,
    }
    if adc is not None:
        params["adc"] = adc
    if fa is not None:
        params["fa"] = fa
    if ad is not None:
        params["ad"] = ad
    if rd is not None:
        params["rd"] = rd
    if cl is not None:
        params["cl"] = cl
    if cp is not None:
        params["cp"] = cp
    if cs is not None:
        params["cs"] = cs
    if value is not None:
        params["value"] = value
    if vector is not None:
        params["vector"] = vector
    if num is not None:
        params["num"] = num
    if modulate is not None:
        params["modulate"] = modulate
    if mask is not None:
        params["mask"] = mask
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tensor2metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Tensor2metricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("adc", None) is not None:
        if not isinstance(params["adc"], str):
            raise StyxValidationError(f'`adc` has the wrong type: Received `{type(params.get("adc", None))}` expected `str | None`')
    if params.get("fa", None) is not None:
        if not isinstance(params["fa"], str):
            raise StyxValidationError(f'`fa` has the wrong type: Received `{type(params.get("fa", None))}` expected `str | None`')
    if params.get("ad", None) is not None:
        if not isinstance(params["ad"], str):
            raise StyxValidationError(f'`ad` has the wrong type: Received `{type(params.get("ad", None))}` expected `str | None`')
    if params.get("rd", None) is not None:
        if not isinstance(params["rd"], str):
            raise StyxValidationError(f'`rd` has the wrong type: Received `{type(params.get("rd", None))}` expected `str | None`')
    if params.get("cl", None) is not None:
        if not isinstance(params["cl"], str):
            raise StyxValidationError(f'`cl` has the wrong type: Received `{type(params.get("cl", None))}` expected `str | None`')
    if params.get("cp", None) is not None:
        if not isinstance(params["cp"], str):
            raise StyxValidationError(f'`cp` has the wrong type: Received `{type(params.get("cp", None))}` expected `str | None`')
    if params.get("cs", None) is not None:
        if not isinstance(params["cs"], str):
            raise StyxValidationError(f'`cs` has the wrong type: Received `{type(params.get("cs", None))}` expected `str | None`')
    if params.get("value", None) is not None:
        if not isinstance(params["value"], str):
            raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str | None`')
    if params.get("vector", None) is not None:
        if not isinstance(params["vector"], str):
            raise StyxValidationError(f'`vector` has the wrong type: Received `{type(params.get("vector", None))}` expected `str | None`')
    if params.get("num", None) is not None:
        if not isinstance(params["num"], list):
            raise StyxValidationError(f'`num` has the wrong type: Received `{type(params.get("num", None))}` expected `list[int] | None`')
        for e in params["num"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`num` has the wrong type: Received `{type(params.get("num", None))}` expected `list[int] | None`')
    if params.get("modulate", None) is not None:
        if not isinstance(params["modulate"], str):
            raise StyxValidationError(f'`modulate` has the wrong type: Received `{type(params.get("modulate", None))}` expected `str | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Tensor2metricConfigParamsDict] | None`')
        for e in params["config"]:
            tensor2metric_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tensor", None) is None:
        raise StyxValidationError("`tensor` must not be None")
    if not isinstance(params["tensor"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tensor` has the wrong type: Received `{type(params.get("tensor", None))}` expected `InputPathType`')


def tensor2metric_cargs(
    params: Tensor2metricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tensor2metric")
    if params.get("adc", None) is not None:
        cargs.extend([
            "-adc",
            params.get("adc", None)
        ])
    if params.get("fa", None) is not None:
        cargs.extend([
            "-fa",
            params.get("fa", None)
        ])
    if params.get("ad", None) is not None:
        cargs.extend([
            "-ad",
            params.get("ad", None)
        ])
    if params.get("rd", None) is not None:
        cargs.extend([
            "-rd",
            params.get("rd", None)
        ])
    if params.get("cl", None) is not None:
        cargs.extend([
            "-cl",
            params.get("cl", None)
        ])
    if params.get("cp", None) is not None:
        cargs.extend([
            "-cp",
            params.get("cp", None)
        ])
    if params.get("cs", None) is not None:
        cargs.extend([
            "-cs",
            params.get("cs", None)
        ])
    if params.get("value", None) is not None:
        cargs.extend([
            "-value",
            params.get("value", None)
        ])
    if params.get("vector", None) is not None:
        cargs.extend([
            "-vector",
            params.get("vector", None)
        ])
    if params.get("num", None) is not None:
        cargs.extend([
            "-num",
            *map(str, params.get("num", None))
        ])
    if params.get("modulate", None) is not None:
        cargs.extend([
            "-modulate",
            params.get("modulate", None)
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tensor2metric_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tensor", None)))
    return cargs


def tensor2metric_outputs(
    params: Tensor2metricParamsDict,
    execution: Execution,
) -> Tensor2metricOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Tensor2metricOutputs(
        root=execution.output_file("."),
        adc=execution.output_file(params.get("adc", None)) if (params.get("adc") is not None) else None,
        fa=execution.output_file(params.get("fa", None)) if (params.get("fa") is not None) else None,
        ad=execution.output_file(params.get("ad", None)) if (params.get("ad") is not None) else None,
        rd=execution.output_file(params.get("rd", None)) if (params.get("rd") is not None) else None,
        cl=execution.output_file(params.get("cl", None)) if (params.get("cl") is not None) else None,
        cp=execution.output_file(params.get("cp", None)) if (params.get("cp") is not None) else None,
        cs=execution.output_file(params.get("cs", None)) if (params.get("cs") is not None) else None,
        value=execution.output_file(params.get("value", None)) if (params.get("value") is not None) else None,
        vector=execution.output_file(params.get("vector", None)) if (params.get("vector") is not None) else None,
    )
    return ret


def tensor2metric_execute(
    params: Tensor2metricParamsDict,
    runner: Runner | None = None,
) -> Tensor2metricOutputs:
    """
    tensor2metric
    
    Generate maps of tensor-derived parameters.
    
    
    
    References:
    
    Basser, P. J.; Mattiello, J. & Lebihan, D. MR diffusion tensor spectroscopy
    and imaging. Biophysical Journal, 1994, 66, 259-267
    
    Westin, C. F.; Peled, S.; Gudbjartsson, H.; Kikinis, R. & Jolesz, F. A.
    Geometrical diffusion measures for MRI from tensor basis analysis. Proc Intl
    Soc Mag Reson Med, 1997, 5, 1742.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tensor2metricOutputs`).
    """
    tensor2metric_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TENSOR2METRIC_METADATA)
    params = execution.params(params)
    cargs = tensor2metric_cargs(params, execution)
    ret = tensor2metric_outputs(params, execution)
    execution.run(cargs)
    return ret


def tensor2metric(
    tensor: InputPathType,
    adc: str | None = None,
    fa: str | None = None,
    ad: str | None = None,
    rd: str | None = None,
    cl: str | None = None,
    cp: str | None = None,
    cs: str | None = None,
    value: str | None = None,
    vector: str | None = None,
    num: list[int] | None = None,
    modulate: str | None = None,
    mask: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Tensor2metricConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Tensor2metricOutputs:
    """
    tensor2metric
    
    Generate maps of tensor-derived parameters.
    
    
    
    References:
    
    Basser, P. J.; Mattiello, J. & Lebihan, D. MR diffusion tensor spectroscopy
    and imaging. Biophysical Journal, 1994, 66, 259-267
    
    Westin, C. F.; Peled, S.; Gudbjartsson, H.; Kikinis, R. & Jolesz, F. A.
    Geometrical diffusion measures for MRI from tensor basis analysis. Proc Intl
    Soc Mag Reson Med, 1997, 5, 1742.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tensor: the input tensor image.
        adc: compute the mean apparent diffusion coefficient (ADC) of the\
            diffusion tensor. (sometimes also referred to as the mean diffusivity\
            (MD)).
        fa: compute the fractional anisotropy (FA) of the diffusion tensor.
        ad: compute the axial diffusivity (AD) of the diffusion tensor.\
            (equivalent to the principal eigenvalue).
        rd: compute the radial diffusivity (RD) of the diffusion tensor.\
            (equivalent to the mean of the two non-principal eigenvalues).
        cl: compute the linearity metric of the diffusion tensor. (one of the\
            three Westin shape metrics).
        cp: compute the planarity metric of the diffusion tensor. (one of the\
            three Westin shape metrics).
        cs: compute the sphericity metric of the diffusion tensor. (one of the\
            three Westin shape metrics).
        value: compute the selected eigenvalue(s) of the diffusion tensor.
        vector: compute the selected eigenvector(s) of the diffusion tensor.
        num: specify the desired eigenvalue/eigenvector(s). Note that several\
            eigenvalues can be specified as a number sequence. For example, '1,3'\
            specifies the principal (1) and minor (3) eigenvalues/eigenvectors\
            (default = 1).
        modulate: specify how to modulate the magnitude of the eigenvectors.\
            Valid choices are: none, FA, eigval (default = FA).
        mask: only perform computation within the specified binary brain mask\
            image.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tensor2metricOutputs`).
    """
    params = tensor2metric_params(
        adc=adc,
        fa=fa,
        ad=ad,
        rd=rd,
        cl=cl,
        cp=cp,
        cs=cs,
        value=value,
        vector=vector,
        num=num,
        modulate=modulate,
        mask=mask,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tensor=tensor,
    )
    return tensor2metric_execute(params, runner)


__all__ = [
    "TENSOR2METRIC_METADATA",
    "Tensor2metricConfigParamsDict",
    "Tensor2metricConfigParamsDictTagged",
    "Tensor2metricOutputs",
    "Tensor2metricParamsDict",
    "Tensor2metricParamsDictTagged",
    "tensor2metric",
    "tensor2metric_config",
    "tensor2metric_execute",
    "tensor2metric_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCK2CONNECTOME_METADATA = Metadata(
    id="a66d5312bce1284177c4cf9ad60b3718c4a1a2b0.boutiques",
    name="tck2connectome",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Tck2connectomeConfigParamsDictNoTag = typing.TypedDict('_Tck2connectomeConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Tck2connectomeConfigParamsDictTagged = typing.TypedDict('Tck2connectomeConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Tck2connectomeConfigParamsDict = _Tck2connectomeConfigParamsDictNoTag | Tck2connectomeConfigParamsDictTagged


_Tck2connectomeParamsDictNoTag = typing.TypedDict('_Tck2connectomeParamsDictNoTag', {
    "assignment_end_voxels": bool,
    "assignment_radial_search": typing.NotRequired[float | None],
    "assignment_reverse_search": typing.NotRequired[float | None],
    "assignment_forward_search": typing.NotRequired[float | None],
    "assignment_all_voxels": bool,
    "scale_length": bool,
    "scale_invlength": bool,
    "scale_invnodevol": bool,
    "scale_file": typing.NotRequired[InputPathType | None],
    "symmetric": bool,
    "zero_diagonal": bool,
    "stat_edge": typing.NotRequired[str | None],
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "keep_unassigned": bool,
    "out_assignments": typing.NotRequired[str | None],
    "vector": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Tck2connectomeConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks_in": InputPathType,
    "nodes_in": InputPathType,
    "connectome_out": str,
})
Tck2connectomeParamsDictTagged = typing.TypedDict('Tck2connectomeParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tck2connectome"],
    "assignment_end_voxels": bool,
    "assignment_radial_search": typing.NotRequired[float | None],
    "assignment_reverse_search": typing.NotRequired[float | None],
    "assignment_forward_search": typing.NotRequired[float | None],
    "assignment_all_voxels": bool,
    "scale_length": bool,
    "scale_invlength": bool,
    "scale_invnodevol": bool,
    "scale_file": typing.NotRequired[InputPathType | None],
    "symmetric": bool,
    "zero_diagonal": bool,
    "stat_edge": typing.NotRequired[str | None],
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "keep_unassigned": bool,
    "out_assignments": typing.NotRequired[str | None],
    "vector": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Tck2connectomeConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks_in": InputPathType,
    "nodes_in": InputPathType,
    "connectome_out": str,
})
Tck2connectomeParamsDict = _Tck2connectomeParamsDictNoTag | Tck2connectomeParamsDictTagged


def tck2connectome_config(
    key: str,
    value: str,
) -> Tck2connectomeConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tck2connectome_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Tck2connectomeConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tck2connectome_config_cargs(
    params: Tck2connectomeConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Tck2connectomeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Tck2connectomeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    connectome_out: OutputPathType
    """the output .csv file containing edge weights"""
    out_assignments: OutputPathType | None
    """output the node assignments of each streamline to a file; this can be
    used subsequently e.g. by the command connectome2tck """


def tck2connectome_params(
    tracks_in: InputPathType,
    nodes_in: InputPathType,
    connectome_out: str,
    assignment_end_voxels: bool = False,
    assignment_radial_search: float | None = None,
    assignment_reverse_search: float | None = None,
    assignment_forward_search: float | None = None,
    assignment_all_voxels: bool = False,
    scale_length: bool = False,
    scale_invlength: bool = False,
    scale_invnodevol: bool = False,
    scale_file: InputPathType | None = None,
    symmetric: bool = False,
    zero_diagonal: bool = False,
    stat_edge: str | None = None,
    tck_weights_in: InputPathType | None = None,
    keep_unassigned: bool = False,
    out_assignments: str | None = None,
    vector: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Tck2connectomeConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Tck2connectomeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tracks_in: the input track file.
        nodes_in: the input node parcellation image.
        connectome_out: the output .csv file containing edge weights.
        assignment_end_voxels: use a simple voxel lookup value at each\
            streamline endpoint.
        assignment_radial_search: perform a radial search from each streamline\
            endpoint to locate the nearest node. Argument is the maximum radius in\
            mm; if no node is found within this radius, the streamline endpoint is\
            not assigned to any node. Default search distance is 4mm.
        assignment_reverse_search: traverse from each streamline endpoint\
            inwards along the streamline, in search of the last node traversed by\
            the streamline. Argument is the maximum traversal length in mm (set to\
            0 to allow search to continue to the streamline midpoint).
        assignment_forward_search: project the streamline forwards from the\
            endpoint in search of a parcellation node voxel. Argument is the\
            maximum traversal length in mm.
        assignment_all_voxels: assign the streamline to all nodes it intersects\
            along its length (note that this means a streamline may be assigned to\
            more than two nodes, or indeed none at all).
        scale_length: scale each contribution to the connectome edge by the\
            length of the streamline.
        scale_invlength: scale each contribution to the connectome edge by the\
            inverse of the streamline length.
        scale_invnodevol: scale each contribution to the connectome edge by the\
            inverse of the two node volumes.
        scale_file: scale each contribution to the connectome edge according to\
            the values in a vector file.
        symmetric: Make matrices symmetric on output.
        zero_diagonal: Set matrix diagonal to zero on output.
        stat_edge: statistic for combining the values from all streamlines in\
            an edge into a single scale value for that edge (options are:\
            sum,mean,min,max; default=sum).
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        keep_unassigned: By default, the program discards the information\
            regarding those streamlines that are not successfully assigned to a\
            node pair. Set this option to keep these values (will be the first\
            row/column in the output matrix).
        out_assignments: output the node assignments of each streamline to a\
            file; this can be used subsequently e.g. by the command connectome2tck.
        vector: output a vector representing connectivities from a given seed\
            point to target nodes, rather than a matrix of node-node connectivities.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tck2connectome",
        "assignment_end_voxels": assignment_end_voxels,
        "assignment_all_voxels": assignment_all_voxels,
        "scale_length": scale_length,
        "scale_invlength": scale_invlength,
        "scale_invnodevol": scale_invnodevol,
        "symmetric": symmetric,
        "zero_diagonal": zero_diagonal,
        "keep_unassigned": keep_unassigned,
        "vector": vector,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tracks_in": tracks_in,
        "nodes_in": nodes_in,
        "connectome_out": connectome_out,
    }
    if assignment_radial_search is not None:
        params["assignment_radial_search"] = assignment_radial_search
    if assignment_reverse_search is not None:
        params["assignment_reverse_search"] = assignment_reverse_search
    if assignment_forward_search is not None:
        params["assignment_forward_search"] = assignment_forward_search
    if scale_file is not None:
        params["scale_file"] = scale_file
    if stat_edge is not None:
        params["stat_edge"] = stat_edge
    if tck_weights_in is not None:
        params["tck_weights_in"] = tck_weights_in
    if out_assignments is not None:
        params["out_assignments"] = out_assignments
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tck2connectome_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Tck2connectomeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("assignment_end_voxels", False) is None:
        raise StyxValidationError("`assignment_end_voxels` must not be None")
    if not isinstance(params["assignment_end_voxels"], bool):
        raise StyxValidationError(f'`assignment_end_voxels` has the wrong type: Received `{type(params.get("assignment_end_voxels", False))}` expected `bool`')
    if params.get("assignment_radial_search", None) is not None:
        if not isinstance(params["assignment_radial_search"], (float, int)):
            raise StyxValidationError(f'`assignment_radial_search` has the wrong type: Received `{type(params.get("assignment_radial_search", None))}` expected `float | None`')
    if params.get("assignment_reverse_search", None) is not None:
        if not isinstance(params["assignment_reverse_search"], (float, int)):
            raise StyxValidationError(f'`assignment_reverse_search` has the wrong type: Received `{type(params.get("assignment_reverse_search", None))}` expected `float | None`')
    if params.get("assignment_forward_search", None) is not None:
        if not isinstance(params["assignment_forward_search"], (float, int)):
            raise StyxValidationError(f'`assignment_forward_search` has the wrong type: Received `{type(params.get("assignment_forward_search", None))}` expected `float | None`')
    if params.get("assignment_all_voxels", False) is None:
        raise StyxValidationError("`assignment_all_voxels` must not be None")
    if not isinstance(params["assignment_all_voxels"], bool):
        raise StyxValidationError(f'`assignment_all_voxels` has the wrong type: Received `{type(params.get("assignment_all_voxels", False))}` expected `bool`')
    if params.get("scale_length", False) is None:
        raise StyxValidationError("`scale_length` must not be None")
    if not isinstance(params["scale_length"], bool):
        raise StyxValidationError(f'`scale_length` has the wrong type: Received `{type(params.get("scale_length", False))}` expected `bool`')
    if params.get("scale_invlength", False) is None:
        raise StyxValidationError("`scale_invlength` must not be None")
    if not isinstance(params["scale_invlength"], bool):
        raise StyxValidationError(f'`scale_invlength` has the wrong type: Received `{type(params.get("scale_invlength", False))}` expected `bool`')
    if params.get("scale_invnodevol", False) is None:
        raise StyxValidationError("`scale_invnodevol` must not be None")
    if not isinstance(params["scale_invnodevol"], bool):
        raise StyxValidationError(f'`scale_invnodevol` has the wrong type: Received `{type(params.get("scale_invnodevol", False))}` expected `bool`')
    if params.get("scale_file", None) is not None:
        if not isinstance(params["scale_file"], (pathlib.Path, str)):
            raise StyxValidationError(f'`scale_file` has the wrong type: Received `{type(params.get("scale_file", None))}` expected `InputPathType | None`')
    if params.get("symmetric", False) is None:
        raise StyxValidationError("`symmetric` must not be None")
    if not isinstance(params["symmetric"], bool):
        raise StyxValidationError(f'`symmetric` has the wrong type: Received `{type(params.get("symmetric", False))}` expected `bool`')
    if params.get("zero_diagonal", False) is None:
        raise StyxValidationError("`zero_diagonal` must not be None")
    if not isinstance(params["zero_diagonal"], bool):
        raise StyxValidationError(f'`zero_diagonal` has the wrong type: Received `{type(params.get("zero_diagonal", False))}` expected `bool`')
    if params.get("stat_edge", None) is not None:
        if not isinstance(params["stat_edge"], str):
            raise StyxValidationError(f'`stat_edge` has the wrong type: Received `{type(params.get("stat_edge", None))}` expected `str | None`')
    if params.get("tck_weights_in", None) is not None:
        if not isinstance(params["tck_weights_in"], (pathlib.Path, str)):
            raise StyxValidationError(f'`tck_weights_in` has the wrong type: Received `{type(params.get("tck_weights_in", None))}` expected `InputPathType | None`')
    if params.get("keep_unassigned", False) is None:
        raise StyxValidationError("`keep_unassigned` must not be None")
    if not isinstance(params["keep_unassigned"], bool):
        raise StyxValidationError(f'`keep_unassigned` has the wrong type: Received `{type(params.get("keep_unassigned", False))}` expected `bool`')
    if params.get("out_assignments", None) is not None:
        if not isinstance(params["out_assignments"], str):
            raise StyxValidationError(f'`out_assignments` has the wrong type: Received `{type(params.get("out_assignments", None))}` expected `str | None`')
    if params.get("vector", False) is None:
        raise StyxValidationError("`vector` must not be None")
    if not isinstance(params["vector"], bool):
        raise StyxValidationError(f'`vector` has the wrong type: Received `{type(params.get("vector", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Tck2connectomeConfigParamsDict] | None`')
        for e in params["config"]:
            tck2connectome_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tracks_in", None) is None:
        raise StyxValidationError("`tracks_in` must not be None")
    if not isinstance(params["tracks_in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks_in` has the wrong type: Received `{type(params.get("tracks_in", None))}` expected `InputPathType`')
    if params.get("nodes_in", None) is None:
        raise StyxValidationError("`nodes_in` must not be None")
    if not isinstance(params["nodes_in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`nodes_in` has the wrong type: Received `{type(params.get("nodes_in", None))}` expected `InputPathType`')
    if params.get("connectome_out", None) is None:
        raise StyxValidationError("`connectome_out` must not be None")
    if not isinstance(params["connectome_out"], str):
        raise StyxValidationError(f'`connectome_out` has the wrong type: Received `{type(params.get("connectome_out", None))}` expected `str`')


def tck2connectome_cargs(
    params: Tck2connectomeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tck2connectome")
    if params.get("assignment_end_voxels", False):
        cargs.append("-assignment_end_voxels")
    if params.get("assignment_radial_search", None) is not None:
        cargs.extend([
            "-assignment_radial_search",
            str(params.get("assignment_radial_search", None))
        ])
    if params.get("assignment_reverse_search", None) is not None:
        cargs.extend([
            "-assignment_reverse_search",
            str(params.get("assignment_reverse_search", None))
        ])
    if params.get("assignment_forward_search", None) is not None:
        cargs.extend([
            "-assignment_forward_search",
            str(params.get("assignment_forward_search", None))
        ])
    if params.get("assignment_all_voxels", False):
        cargs.append("-assignment_all_voxels")
    if params.get("scale_length", False):
        cargs.append("-scale_length")
    if params.get("scale_invlength", False):
        cargs.append("-scale_invlength")
    if params.get("scale_invnodevol", False):
        cargs.append("-scale_invnodevol")
    if params.get("scale_file", None) is not None:
        cargs.extend([
            "-scale_file",
            execution.input_file(params.get("scale_file", None))
        ])
    if params.get("symmetric", False):
        cargs.append("-symmetric")
    if params.get("zero_diagonal", False):
        cargs.append("-zero_diagonal")
    if params.get("stat_edge", None) is not None:
        cargs.extend([
            "-stat_edge",
            params.get("stat_edge", None)
        ])
    if params.get("tck_weights_in", None) is not None:
        cargs.extend([
            "-tck_weights_in",
            execution.input_file(params.get("tck_weights_in", None))
        ])
    if params.get("keep_unassigned", False):
        cargs.append("-keep_unassigned")
    if params.get("out_assignments", None) is not None:
        cargs.extend([
            "-out_assignments",
            params.get("out_assignments", None)
        ])
    if params.get("vector", False):
        cargs.append("-vector")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tck2connectome_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tracks_in", None)))
    cargs.append(execution.input_file(params.get("nodes_in", None)))
    cargs.append(params.get("connectome_out", None))
    return cargs


def tck2connectome_outputs(
    params: Tck2connectomeParamsDict,
    execution: Execution,
) -> Tck2connectomeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Tck2connectomeOutputs(
        root=execution.output_file("."),
        connectome_out=execution.output_file(params.get("connectome_out", None)),
        out_assignments=execution.output_file(params.get("out_assignments", None)) if (params.get("out_assignments") is not None) else None,
    )
    return ret


def tck2connectome_execute(
    params: Tck2connectomeParamsDict,
    runner: Runner | None = None,
) -> Tck2connectomeOutputs:
    """
    tck2connectome
    
    Generate a connectome matrix from a streamlines file and a node parcellation
    image.
    
    
    
    References:
    
    If using the default streamline-parcel assignment mechanism (or
    -assignment_radial_search option): Smith, R. E.; Tournier, J.-D.; Calamante,
    F. & Connelly, A. The effects of SIFT on the reproducibility and biological
    accuracy of the structural connectome. NeuroImage, 2015, 104, 253-265
    
    If using -scale_invlength or -scale_invnodevol options: Hagmann, P.;
    Cammoun, L.; Gigandet, X.; Meuli, R.; Honey, C.; Wedeen, V. & Sporns, O.
    Mapping the Structural Core of Human Cerebral Cortex. PLoS Biology 6(7),
    e159.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tck2connectomeOutputs`).
    """
    tck2connectome_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCK2CONNECTOME_METADATA)
    params = execution.params(params)
    cargs = tck2connectome_cargs(params, execution)
    ret = tck2connectome_outputs(params, execution)
    execution.run(cargs)
    return ret


def tck2connectome(
    tracks_in: InputPathType,
    nodes_in: InputPathType,
    connectome_out: str,
    assignment_end_voxels: bool = False,
    assignment_radial_search: float | None = None,
    assignment_reverse_search: float | None = None,
    assignment_forward_search: float | None = None,
    assignment_all_voxels: bool = False,
    scale_length: bool = False,
    scale_invlength: bool = False,
    scale_invnodevol: bool = False,
    scale_file: InputPathType | None = None,
    symmetric: bool = False,
    zero_diagonal: bool = False,
    stat_edge: str | None = None,
    tck_weights_in: InputPathType | None = None,
    keep_unassigned: bool = False,
    out_assignments: str | None = None,
    vector: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Tck2connectomeConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Tck2connectomeOutputs:
    """
    tck2connectome
    
    Generate a connectome matrix from a streamlines file and a node parcellation
    image.
    
    
    
    References:
    
    If using the default streamline-parcel assignment mechanism (or
    -assignment_radial_search option): Smith, R. E.; Tournier, J.-D.; Calamante,
    F. & Connelly, A. The effects of SIFT on the reproducibility and biological
    accuracy of the structural connectome. NeuroImage, 2015, 104, 253-265
    
    If using -scale_invlength or -scale_invnodevol options: Hagmann, P.;
    Cammoun, L.; Gigandet, X.; Meuli, R.; Honey, C.; Wedeen, V. & Sporns, O.
    Mapping the Structural Core of Human Cerebral Cortex. PLoS Biology 6(7),
    e159.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks_in: the input track file.
        nodes_in: the input node parcellation image.
        connectome_out: the output .csv file containing edge weights.
        assignment_end_voxels: use a simple voxel lookup value at each\
            streamline endpoint.
        assignment_radial_search: perform a radial search from each streamline\
            endpoint to locate the nearest node. Argument is the maximum radius in\
            mm; if no node is found within this radius, the streamline endpoint is\
            not assigned to any node. Default search distance is 4mm.
        assignment_reverse_search: traverse from each streamline endpoint\
            inwards along the streamline, in search of the last node traversed by\
            the streamline. Argument is the maximum traversal length in mm (set to\
            0 to allow search to continue to the streamline midpoint).
        assignment_forward_search: project the streamline forwards from the\
            endpoint in search of a parcellation node voxel. Argument is the\
            maximum traversal length in mm.
        assignment_all_voxels: assign the streamline to all nodes it intersects\
            along its length (note that this means a streamline may be assigned to\
            more than two nodes, or indeed none at all).
        scale_length: scale each contribution to the connectome edge by the\
            length of the streamline.
        scale_invlength: scale each contribution to the connectome edge by the\
            inverse of the streamline length.
        scale_invnodevol: scale each contribution to the connectome edge by the\
            inverse of the two node volumes.
        scale_file: scale each contribution to the connectome edge according to\
            the values in a vector file.
        symmetric: Make matrices symmetric on output.
        zero_diagonal: Set matrix diagonal to zero on output.
        stat_edge: statistic for combining the values from all streamlines in\
            an edge into a single scale value for that edge (options are:\
            sum,mean,min,max; default=sum).
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        keep_unassigned: By default, the program discards the information\
            regarding those streamlines that are not successfully assigned to a\
            node pair. Set this option to keep these values (will be the first\
            row/column in the output matrix).
        out_assignments: output the node assignments of each streamline to a\
            file; this can be used subsequently e.g. by the command connectome2tck.
        vector: output a vector representing connectivities from a given seed\
            point to target nodes, rather than a matrix of node-node connectivities.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tck2connectomeOutputs`).
    """
    params = tck2connectome_params(
        assignment_end_voxels=assignment_end_voxels,
        assignment_radial_search=assignment_radial_search,
        assignment_reverse_search=assignment_reverse_search,
        assignment_forward_search=assignment_forward_search,
        assignment_all_voxels=assignment_all_voxels,
        scale_length=scale_length,
        scale_invlength=scale_invlength,
        scale_invnodevol=scale_invnodevol,
        scale_file=scale_file,
        symmetric=symmetric,
        zero_diagonal=zero_diagonal,
        stat_edge=stat_edge,
        tck_weights_in=tck_weights_in,
        keep_unassigned=keep_unassigned,
        out_assignments=out_assignments,
        vector=vector,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tracks_in=tracks_in,
        nodes_in=nodes_in,
        connectome_out=connectome_out,
    )
    return tck2connectome_execute(params, runner)


__all__ = [
    "TCK2CONNECTOME_METADATA",
    "Tck2connectomeConfigParamsDict",
    "Tck2connectomeConfigParamsDictTagged",
    "Tck2connectomeOutputs",
    "Tck2connectomeParamsDict",
    "Tck2connectomeParamsDictTagged",
    "tck2connectome",
    "tck2connectome_config",
    "tck2connectome_execute",
    "tck2connectome_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DWIBIASCORRECT_METADATA = Metadata(
    id="b6c72322d3d00ba26448970257e9f0b213c34953.boutiques",
    name="dwibiascorrect",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DwibiascorrectFslgradParamsDictNoTag = typing.TypedDict('_DwibiascorrectFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwibiascorrectFslgradParamsDictTagged = typing.TypedDict('DwibiascorrectFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwibiascorrectFslgradParamsDict = _DwibiascorrectFslgradParamsDictNoTag | DwibiascorrectFslgradParamsDictTagged


_DwibiascorrectConfigParamsDictNoTag = typing.TypedDict('_DwibiascorrectConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DwibiascorrectConfigParamsDictTagged = typing.TypedDict('DwibiascorrectConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DwibiascorrectConfigParamsDict = _DwibiascorrectConfigParamsDictNoTag | DwibiascorrectConfigParamsDictTagged


_DwibiascorrectParamsDictNoTag = typing.TypedDict('_DwibiascorrectParamsDictNoTag', {
    "algorithm": str,
    "input_image": InputPathType,
    "output_image": str,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwibiascorrectFslgradParamsDict | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "bias_image": typing.NotRequired[InputPathType | None],
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[DwibiascorrectConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "ants_b": typing.NotRequired[str | None],
    "ants_c": typing.NotRequired[str | None],
    "ants_s": typing.NotRequired[str | None],
})
DwibiascorrectParamsDictTagged = typing.TypedDict('DwibiascorrectParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dwibiascorrect"],
    "algorithm": str,
    "input_image": InputPathType,
    "output_image": str,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwibiascorrectFslgradParamsDict | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "bias_image": typing.NotRequired[InputPathType | None],
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[DwibiascorrectConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "ants_b": typing.NotRequired[str | None],
    "ants_c": typing.NotRequired[str | None],
    "ants_s": typing.NotRequired[str | None],
})
DwibiascorrectParamsDict = _DwibiascorrectParamsDictNoTag | DwibiascorrectParamsDictTagged


def dwibiascorrect_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> DwibiascorrectFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dwibiascorrect_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwibiascorrectFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dwibiascorrect_fslgrad_cargs(
    params: DwibiascorrectFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def dwibiascorrect_config(
    key: str,
    value: str,
) -> DwibiascorrectConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dwibiascorrect_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwibiascorrectConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dwibiascorrect_config_cargs(
    params: DwibiascorrectConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DwibiascorrectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DwibiascorrectParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """File containing the corrected image series"""
    bias_image_file: OutputPathType | None
    """File containing the estimated bias field"""


def dwibiascorrect_params(
    algorithm: str,
    input_image: InputPathType,
    output_image: str,
    grad: InputPathType | None = None,
    fslgrad: DwibiascorrectFslgradParamsDict | None = None,
    mask_image: InputPathType | None = None,
    bias_image: InputPathType | None = None,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[DwibiascorrectConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    ants_b: str | None = None,
    ants_c: str | None = None,
    ants_s: str | None = None,
) -> DwibiascorrectParamsDictTagged:
    """
    Build parameters.
    
    Args:
        algorithm: Select the algorithm to be used for bias correction. Options\
            are: ants, fsl.
        input_image: The input image series to be corrected.
        output_image: The output corrected image series.
        grad: Provide the diffusion gradient table in MRtrix format.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        mask_image: Manually provide a mask image for bias field estimation.
        bias_image: Output the estimated bias field.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        ants_b: N4BiasFieldCorrection option -b (initial mesh resolution in mm,\
            spline order).
        ants_c: N4BiasFieldCorrection option -c (number of iterations,\
            convergence threshold).
        ants_s: N4BiasFieldCorrection option -s (shrink-factor applied to\
            spatial dimensions).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dwibiascorrect",
        "algorithm": algorithm,
        "input_image": input_image,
        "output_image": output_image,
        "nocleanup": nocleanup,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
    }
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if mask_image is not None:
        params["mask_image"] = mask_image
    if bias_image is not None:
        params["bias_image"] = bias_image
    if scratch_dir is not None:
        params["scratch_dir"] = scratch_dir
    if continue_scratch_dir is not None:
        params["continue_scratch_dir"] = continue_scratch_dir
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    if ants_b is not None:
        params["ants_b"] = ants_b
    if ants_c is not None:
        params["ants_c"] = ants_c
    if ants_s is not None:
        params["ants_s"] = ants_s
    return params


def dwibiascorrect_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwibiascorrectParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("algorithm", None) is None:
        raise StyxValidationError("`algorithm` must not be None")
    if not isinstance(params["algorithm"], str):
        raise StyxValidationError(f'`algorithm` has the wrong type: Received `{type(params.get("algorithm", None))}` expected `str`')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dwibiascorrect_fslgrad_validate(params["fslgrad"])
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("bias_image", None) is not None:
        if not isinstance(params["bias_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`bias_image` has the wrong type: Received `{type(params.get("bias_image", None))}` expected `InputPathType | None`')
    if params.get("nocleanup", False) is None:
        raise StyxValidationError("`nocleanup` must not be None")
    if not isinstance(params["nocleanup"], bool):
        raise StyxValidationError(f'`nocleanup` has the wrong type: Received `{type(params.get("nocleanup", False))}` expected `bool`')
    if params.get("scratch_dir", None) is not None:
        if not isinstance(params["scratch_dir"], (pathlib.Path, str)):
            raise StyxValidationError(f'`scratch_dir` has the wrong type: Received `{type(params.get("scratch_dir", None))}` expected `InputPathType | None`')
    if params.get("continue_scratch_dir", None) is not None:
        if not isinstance(params["continue_scratch_dir"], list):
            raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
        if len(params["continue_scratch_dir"]) != 2:
            raise StyxValidationError("Parameter `continue_scratch_dir` must contain exactly 2 elements")
        for e in params["continue_scratch_dir"]:
            if not isinstance(e, (pathlib.Path, str)):
                raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], (float, int)):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `float | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DwibiascorrectConfigParamsDict] | None`')
        for e in params["config"]:
            dwibiascorrect_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("ants_b", None) is not None:
        if not isinstance(params["ants_b"], str):
            raise StyxValidationError(f'`ants_b` has the wrong type: Received `{type(params.get("ants_b", None))}` expected `str | None`')
    if params.get("ants_c", None) is not None:
        if not isinstance(params["ants_c"], str):
            raise StyxValidationError(f'`ants_c` has the wrong type: Received `{type(params.get("ants_c", None))}` expected `str | None`')
    if params.get("ants_s", None) is not None:
        if not isinstance(params["ants_s"], str):
            raise StyxValidationError(f'`ants_s` has the wrong type: Received `{type(params.get("ants_s", None))}` expected `str | None`')


def dwibiascorrect_cargs(
    params: DwibiascorrectParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dwibiascorrect")
    cargs.append(params.get("algorithm", None))
    cargs.append(execution.input_file(params.get("input_image", None)))
    cargs.append(params.get("output_image", None))
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dwibiascorrect_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask_image", None))
        ])
    if params.get("bias_image", None) is not None:
        cargs.extend([
            "-bias",
            execution.input_file(params.get("bias_image", None))
        ])
    if params.get("nocleanup", False):
        cargs.append("-nocleanup")
    if params.get("scratch_dir", None) is not None:
        cargs.extend([
            "-scratch",
            execution.input_file(params.get("scratch_dir", None))
        ])
    if params.get("continue_scratch_dir", None) is not None:
        cargs.extend([
            "-continue",
            *[execution.input_file(f) for f in params.get("continue_scratch_dir", None)]
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dwibiascorrect_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    if params.get("ants_b", None) is not None:
        cargs.extend([
            "-ants.b",
            params.get("ants_b", None)
        ])
    if params.get("ants_c", None) is not None:
        cargs.extend([
            "-ants.c",
            params.get("ants_c", None)
        ])
    if params.get("ants_s", None) is not None:
        cargs.extend([
            "-ants.s",
            params.get("ants_s", None)
        ])
    return cargs


def dwibiascorrect_outputs(
    params: DwibiascorrectParamsDict,
    execution: Execution,
) -> DwibiascorrectOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DwibiascorrectOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(params.get("output_image", None)),
        bias_image_file=execution.output_file(pathlib.Path(params.get("bias_image", None)).name) if (params.get("bias_image") is not None) else None,
    )
    return ret


def dwibiascorrect_execute(
    params: DwibiascorrectParamsDict,
    runner: Runner | None = None,
) -> DwibiascorrectOutputs:
    """
    dwibiascorrect
    
    Perform B1 field inhomogeneity correction for a DWI volume series using
    either ANTs or FSL.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwibiascorrectOutputs`).
    """
    dwibiascorrect_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWIBIASCORRECT_METADATA)
    params = execution.params(params)
    cargs = dwibiascorrect_cargs(params, execution)
    ret = dwibiascorrect_outputs(params, execution)
    execution.run(cargs)
    return ret


def dwibiascorrect(
    algorithm: str,
    input_image: InputPathType,
    output_image: str,
    grad: InputPathType | None = None,
    fslgrad: DwibiascorrectFslgradParamsDict | None = None,
    mask_image: InputPathType | None = None,
    bias_image: InputPathType | None = None,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[DwibiascorrectConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    ants_b: str | None = None,
    ants_c: str | None = None,
    ants_s: str | None = None,
    runner: Runner | None = None,
) -> DwibiascorrectOutputs:
    """
    dwibiascorrect
    
    Perform B1 field inhomogeneity correction for a DWI volume series using
    either ANTs or FSL.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        algorithm: Select the algorithm to be used for bias correction. Options\
            are: ants, fsl.
        input_image: The input image series to be corrected.
        output_image: The output corrected image series.
        grad: Provide the diffusion gradient table in MRtrix format.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        mask_image: Manually provide a mask image for bias field estimation.
        bias_image: Output the estimated bias field.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        ants_b: N4BiasFieldCorrection option -b (initial mesh resolution in mm,\
            spline order).
        ants_c: N4BiasFieldCorrection option -c (number of iterations,\
            convergence threshold).
        ants_s: N4BiasFieldCorrection option -s (shrink-factor applied to\
            spatial dimensions).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwibiascorrectOutputs`).
    """
    params = dwibiascorrect_params(
        algorithm=algorithm,
        input_image=input_image,
        output_image=output_image,
        grad=grad,
        fslgrad=fslgrad,
        mask_image=mask_image,
        bias_image=bias_image,
        nocleanup=nocleanup,
        scratch_dir=scratch_dir,
        continue_scratch_dir=continue_scratch_dir,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        ants_b=ants_b,
        ants_c=ants_c,
        ants_s=ants_s,
    )
    return dwibiascorrect_execute(params, runner)


__all__ = [
    "DWIBIASCORRECT_METADATA",
    "DwibiascorrectConfigParamsDict",
    "DwibiascorrectConfigParamsDictTagged",
    "DwibiascorrectFslgradParamsDict",
    "DwibiascorrectFslgradParamsDictTagged",
    "DwibiascorrectOutputs",
    "DwibiascorrectParamsDict",
    "DwibiascorrectParamsDictTagged",
    "dwibiascorrect",
    "dwibiascorrect_config",
    "dwibiascorrect_execute",
    "dwibiascorrect_fslgrad",
    "dwibiascorrect_params",
]
